//
//  AnalogSynthesizerViewController.m
//  AnalogSynthesizer
//
//  Created by 成田 拓郎 on 10/05/23.
//  Copyright 2010 lovelyshark. All rights reserved.
//

#import "AnalogSynthesizerViewController.h"

@implementation AnalogSynthesizerViewController

@synthesize keyboardView;

@synthesize vcoWaveTypeVolumeView;
@synthesize vcoPulseWidthVolumeView;
@synthesize vcoControlLevelVolumeView;

@synthesize vcfCutOffFrequencyVolumeView;
@synthesize vcfResonanceVolumeView;
@synthesize vcfControlLevelVolumeView;

@synthesize vcaControlLevelVolumeView;

@synthesize egParamAttackVolumeView;
@synthesize egParamDecayVolumeView;
@synthesize egParamSustainVolumeView;
@synthesize egParamReleaseVolumeView;

@synthesize lfoWaveTypeVolumeView;
@synthesize lfoPulseWidthVolumeView;
@synthesize lfoFrequencyVolumeView;

@synthesize echoDelayTimeVolumeView;
@synthesize echoDelayLevelVolumeView;

@synthesize synthesizerOutput;

/*
// The designated initializer. Override to perform setup that is required before the view is loaded.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if ((self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil])) {
        // Custom initialization
    }
    return self;
}
*/

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/

//オーディオセッション　割り込みリスナーコールバック関数
static void interruptionListenerCallback(void *inUserData, UInt32 interruptionState)
{
	AnalogSynthesizerViewController *controller = (AnalogSynthesizerViewController *)inUserData;
	
	if (interruptionState == kAudioSessionBeginInterruption)
	{
		//オーディオセッション割り込みの開始
		//再生の停止
		controller.synthesizerOutput.play = NO;
	}
	else if (interruptionState == kAudioSessionEndInterruption)
	{
		//オーディオセッション割り込みの終了
		//オーディオセッションのカテゴリを設定
		UInt32 sessionCategory = kAudioSessionCategory_SoloAmbientSound;
		AudioSessionSetProperty(kAudioSessionProperty_AudioCategory,
								sizeof(sessionCategory),
								&sessionCategory);
		//オーディオセッションのアクティブ化
		AudioSessionSetActive(true);
		//再生の再開
		controller.synthesizerOutput.play = YES;
	}
}

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];

	//AudioUnitの準備
	SynthesizerOutput *synthe = [[SynthesizerOutput alloc] init];
	self.synthesizerOutput = synthe;
	[synthe release];

	//オーディオセッションを初期化
	AudioSessionInitialize(NULL, NULL, interruptionListenerCallback, self);
	//オーディオセッションのカテゴリを設定
	UInt32 sessionCategory = kAudioSessionCategory_SoloAmbientSound;
	AudioSessionSetProperty(kAudioSessionProperty_AudioCategory,
							sizeof(sessionCategory),
							&sessionCategory);
	//オーディオセッションのアクティブ化
	AudioSessionSetActive(true);
	// 再生開始
	self.synthesizerOutput.play = YES;
	
	//UIコントロールの準備
	[keyboardView setAction:@selector(keyboardChanged:)];
	[keyboardView setTarget:self];
	
	[vcoWaveTypeVolumeView setAction:@selector(vcoWaveTypeChanged:)];
	[vcoWaveTypeVolumeView setTarget:self];
	vcoWaveTypeVolumeView.valueKnob = self.synthesizerOutput.vcoWaveType;
	vcoWaveTypeVolumeView.valueMax  = 5;
	[vcoWaveTypeVolumeView rotateView];

	[vcoPulseWidthVolumeView setAction:@selector(vcoPulseWidthChanged:)];
	[vcoPulseWidthVolumeView setTarget:self];
	vcoPulseWidthVolumeView.valueKnob = (int)(self.synthesizerOutput.vcoPulseWidth * 127.0f);
	vcoPulseWidthVolumeView.valueMax = 127;
	[vcoPulseWidthVolumeView rotateView];

	[vcoControlLevelVolumeView setAction:@selector(vcoControlLevelChanged:)];
	[vcoControlLevelVolumeView setTarget:self];
	vcoControlLevelVolumeView.valueKnob = (int)(self.synthesizerOutput.vcoControlLevel * 127.0f);
	vcoControlLevelVolumeView.valueMax = 127;
	[vcoControlLevelVolumeView rotateView];
	
	[vcfCutOffFrequencyVolumeView setAction:@selector(vcfCutOffFrequencyChanged:)];
	[vcfCutOffFrequencyVolumeView setTarget:self];
	vcfCutOffFrequencyVolumeView.valueKnob = (int)(self.synthesizerOutput.vcfCutOffFrequency * 127.0f / 10000.0f);
	vcfCutOffFrequencyVolumeView.valueMax = 127;
	[vcfCutOffFrequencyVolumeView rotateView];

	[vcfResonanceVolumeView setAction:@selector(vcfResonanceChanged:)];
	[vcfResonanceVolumeView setTarget:self];
	vcfResonanceVolumeView.valueKnob = (int)(self.synthesizerOutput.vcfResonance * 127.0f / 2.0f);
	vcfResonanceVolumeView.valueMax = 127;
	[vcfResonanceVolumeView rotateView];

	[vcfControlLevelVolumeView setAction:@selector(vcfControlLevelChanged:)];
	[vcfControlLevelVolumeView setTarget:self];
	vcfControlLevelVolumeView.valueKnob = (int)(self.synthesizerOutput.vcfControlLevel * 127.0f);
	vcfControlLevelVolumeView.valueMax = 127;
	[vcfControlLevelVolumeView rotateView];
	
	[vcaControlLevelVolumeView setAction:@selector(vcaControlLevelChanged:)];
	[vcaControlLevelVolumeView setTarget:self];
	vcaControlLevelVolumeView.valueKnob = (int)(self.synthesizerOutput.vcaControlLevel * 127.0f);
	vcaControlLevelVolumeView.valueMax = 127;
	[vcaControlLevelVolumeView rotateView];
	
	[egParamAttackVolumeView setAction:@selector(egParamAttackChanged:)];
	[egParamAttackVolumeView setTarget:self];
	egParamAttackVolumeView.valueKnob = (int)(self.synthesizerOutput.egParamAttack * 2.0f * 127.0f);
	egParamAttackVolumeView.valueMax = 127;
	[egParamAttackVolumeView rotateView];

	[egParamDecayVolumeView setAction:@selector(egParamDecayChanged:)];
	[egParamDecayVolumeView setTarget:self];
	egParamDecayVolumeView.valueKnob = (int)(self.synthesizerOutput.egParamDecay * 127.0f);
	egParamDecayVolumeView.valueMax = 127;
	[egParamDecayVolumeView rotateView];
	
	[egParamSustainVolumeView setAction:@selector(egParamSustainChanged:)];
	[egParamSustainVolumeView setTarget:self];
	egParamSustainVolumeView.valueKnob = (int)(self.synthesizerOutput.egParamSustain * 127.0f);
	egParamSustainVolumeView.valueMax = 127;
	[egParamSustainVolumeView rotateView];
	
	[egParamReleaseVolumeView setAction:@selector(egParamReleaseChanged:)];
	[egParamReleaseVolumeView setTarget:self];
	egParamReleaseVolumeView.valueKnob = (int)(self.synthesizerOutput.egParamRelease * 127.0f);
	egParamReleaseVolumeView.valueMax = 127;
	[egParamReleaseVolumeView rotateView];
	
	[lfoWaveTypeVolumeView setAction:@selector(lfoWaveTypeChanged:)];
	[lfoWaveTypeVolumeView setTarget:self];
	lfoWaveTypeVolumeView.valueKnob = self.synthesizerOutput.lfoWaveType;
	lfoWaveTypeVolumeView.valueMax = 5;
	[lfoWaveTypeVolumeView rotateView];
	
	[lfoPulseWidthVolumeView setAction:@selector(lfoPulseWidthChanged:)];
	[lfoPulseWidthVolumeView setTarget:self];
	lfoPulseWidthVolumeView.valueKnob = (int)(self.synthesizerOutput.lfoPulseWidth * 127.0f);
	lfoPulseWidthVolumeView.valueMax = 127;
	[lfoPulseWidthVolumeView rotateView];
	
	[lfoFrequencyVolumeView setAction:@selector(lfoFrequencyChanged:)];
	[lfoFrequencyVolumeView setTarget:self];
	lfoFrequencyVolumeView.valueKnob = (int)(self.synthesizerOutput.lfoFrequency *127.0f / 20.0f);
	lfoFrequencyVolumeView.valueMax = 127;
	[lfoFrequencyVolumeView rotateView];
	
	[echoDelayTimeVolumeView setAction:@selector(echoDelayTimeChanged:)];
	[echoDelayTimeVolumeView setTarget:self];
	echoDelayTimeVolumeView.valueKnob = (int)(0.1f * 127.0f);
	echoDelayTimeVolumeView.valueMax = 127;
	[echoDelayTimeVolumeView rotateView];
	
	[echoDelayLevelVolumeView setAction:@selector(echoDelayLevelChanged:)];
	[echoDelayLevelVolumeView setTarget:self];
	echoDelayLevelVolumeView.valueKnob = 0;
	echoDelayLevelVolumeView.valueMax = 127;
	[echoDelayLevelVolumeView rotateView];
}

// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
}

- (void)willAnimateRotationToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration {
	if (toInterfaceOrientation == UIInterfaceOrientationPortrait ||
		toInterfaceOrientation == UIInterfaceOrientationPortraitUpsideDown) {
		// ポートレート モード
		CGRect rect = CGRectMake(0, 0, 48, 48);
		CGFloat x1 = 20 + 24;
		CGFloat x2 = x1 + 56;
		CGFloat x3 = x2 + 56;
		CGFloat x4 = x3 + 56;
		CGFloat x5 = x4 + 56;
		CGFloat y1 = 20 + 24;
		CGFloat y2 = y1 + 64;
		CGFloat y3 = y2 + 64;
		CGFloat y4 = y3 + 64;
		CGFloat y5 = y4 + 64;
		
		self.vcoWaveTypeVolumeView.center = CGPointMake(x1, y1);
		self.vcoWaveTypeVolumeView.bounds = rect;
		self.vcoPulseWidthVolumeView.center = CGPointMake(x2, y1);
		self.vcoPulseWidthVolumeView.bounds = rect;
		self.vcoControlLevelVolumeView.center = CGPointMake(x3, y1);
		self.vcoControlLevelVolumeView.bounds = rect;
		
		self.vcfCutOffFrequencyVolumeView.center = CGPointMake(x1, y2);
		self.vcfCutOffFrequencyVolumeView.bounds = rect;
		self.vcfResonanceVolumeView.center = CGPointMake(x2, y2);
		self.vcfResonanceVolumeView.bounds = rect;
		self.vcfControlLevelVolumeView.center = CGPointMake(x3, y2);
		self.vcfControlLevelVolumeView.bounds = rect;
		
		self.vcaControlLevelVolumeView.center = CGPointMake(x1, y3);
		self.vcaControlLevelVolumeView.bounds = rect;
		
		self.egParamAttackVolumeView.center = CGPointMake(x1, y4);
		self.egParamAttackVolumeView.bounds = rect;
		self.egParamDecayVolumeView.center = CGPointMake(x2, y4);
		self.egParamDecayVolumeView.bounds = rect;
		self.egParamSustainVolumeView.center = CGPointMake(x3, y4);
		self.egParamSustainVolumeView.bounds = rect;
		self.egParamReleaseVolumeView.center = CGPointMake(x4, y4);
		self.egParamReleaseVolumeView.bounds = rect;
		
		self.lfoWaveTypeVolumeView.center = CGPointMake(x1, y5);
		self.lfoWaveTypeVolumeView.bounds = rect;
		self.lfoPulseWidthVolumeView.center = CGPointMake(x2, y5);
		self.lfoPulseWidthVolumeView.bounds = rect;
		self.lfoFrequencyVolumeView.center = CGPointMake(x3, y5);
		self.lfoFrequencyVolumeView.bounds = rect;
		
		self.echoDelayTimeVolumeView.center = CGPointMake(x5, y1);
		self.echoDelayTimeVolumeView.bounds = rect;
		self.echoDelayLevelVolumeView.center = CGPointMake(x5, y2);
		self.echoDelayLevelVolumeView.bounds = rect;
	}
	else {
		// ランドスケープ モード
		CGRect rect = CGRectMake(0, 0, 48, 48);
		CGFloat x1a = 20 + 24;
		CGFloat x2a = x1a + 56;
		CGFloat x3a = x2a + 56;
		CGFloat x4a = x3a + 56;
		CGFloat x1b = x4a + 64;
		CGFloat x2b = x1b + 56;
		CGFloat x3b = x2b + 56;
		CGFloat x4b = x3b + 56;
		CGFloat y1 = 20 + 24;
		CGFloat y2 = y1 + 64;
		CGFloat y3 = y2 + 64;
		
		self.vcoWaveTypeVolumeView.center = CGPointMake(x1a, y1);
		self.vcoWaveTypeVolumeView.bounds = rect;
		self.vcoPulseWidthVolumeView.center = CGPointMake(x2a, y1);
		self.vcoPulseWidthVolumeView.bounds = rect;
		self.vcoControlLevelVolumeView.center = CGPointMake(x3a, y1);
		self.vcoControlLevelVolumeView.bounds = rect;
		
		self.vcfCutOffFrequencyVolumeView.center = CGPointMake(x1a, y2);
		self.vcfCutOffFrequencyVolumeView.bounds = rect;
		self.vcfResonanceVolumeView.center = CGPointMake(x2a, y2);
		self.vcfResonanceVolumeView.bounds = rect;
		self.vcfControlLevelVolumeView.center = CGPointMake(x3a, y2);
		self.vcfControlLevelVolumeView.bounds = rect;
		
		self.vcaControlLevelVolumeView.center = CGPointMake(x1a, y3);
		self.vcaControlLevelVolumeView.bounds = rect;
		
		self.egParamAttackVolumeView.center = CGPointMake(x1b, y1);
		self.egParamAttackVolumeView.bounds = rect;
		self.egParamDecayVolumeView.center = CGPointMake(x2b, y1);
		self.egParamDecayVolumeView.bounds = rect;
		self.egParamSustainVolumeView.center = CGPointMake(x3b, y1);
		self.egParamSustainVolumeView.bounds = rect;
		self.egParamReleaseVolumeView.center = CGPointMake(x4b, y1);
		self.egParamReleaseVolumeView.bounds = rect;
		
		self.lfoWaveTypeVolumeView.center = CGPointMake(x1b, y2);
		self.lfoWaveTypeVolumeView.bounds = rect;
		self.lfoPulseWidthVolumeView.center = CGPointMake(x2b, y2);
		self.lfoPulseWidthVolumeView.bounds = rect;
		self.lfoFrequencyVolumeView.center = CGPointMake(x3b, y2);
		self.lfoFrequencyVolumeView.bounds = rect;

		self.echoDelayTimeVolumeView.center = CGPointMake(x1b, y3);
		self.echoDelayTimeVolumeView.bounds = rect;
		self.echoDelayLevelVolumeView.center = CGPointMake(x2b, y3);
		self.echoDelayLevelVolumeView.bounds = rect;
	}
}

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
	self.keyboardView = nil;

	self.vcoWaveTypeVolumeView = nil;
	self.vcoPulseWidthVolumeView = nil;
	self.vcoControlLevelVolumeView = nil;
	
	self.vcfCutOffFrequencyVolumeView = nil;
	self.vcfResonanceVolumeView = nil;
	self.vcfControlLevelVolumeView = nil;
	
	self.vcaControlLevelVolumeView = nil;
	
	self.egParamAttackVolumeView = nil;
	self.egParamDecayVolumeView = nil;
	self.egParamSustainVolumeView = nil;
	self.egParamReleaseVolumeView = nil;
	
	self.lfoWaveTypeVolumeView = nil;
	self.lfoPulseWidthVolumeView = nil;
	self.lfoFrequencyVolumeView = nil;
}


- (void)dealloc {
	[keyboardView release];

	[vcoWaveTypeVolumeView release];
	[vcoPulseWidthVolumeView release];
	[vcoControlLevelVolumeView release];
	
	[vcfCutOffFrequencyVolumeView release];
	[vcfResonanceVolumeView release];
	[vcfControlLevelVolumeView release];
	
	[vcaControlLevelVolumeView release];
	
	[egParamAttackVolumeView release];
	[egParamDecayVolumeView release];
	[egParamSustainVolumeView release];
	[egParamReleaseVolumeView release];
	
	[lfoWaveTypeVolumeView release];
	[lfoPulseWidthVolumeView release];
	[lfoFrequencyVolumeView release];

	[synthesizerOutput release];
    [super dealloc];
}

-(void)keyboardChanged:(id)sender {
//	NSLog(@"Keyboard Message");
	synthesizerOutput.keyboard = keyboardView.keyDown;
}

-(void)vcoWaveTypeChanged:(id)sender {
	synthesizerOutput.vcoWaveType = vcoWaveTypeVolumeView.valueKnob;
}
-(void)vcoPulseWidthChanged:(id)sender {
	float v = vcoPulseWidthVolumeView.valueKnob;
	float vm = vcoPulseWidthVolumeView.valueMax;
	synthesizerOutput.vcoPulseWidth = v / vm;
}
-(void)vcoControlLevelChanged:(id)sender {
	float v = vcoControlLevelVolumeView.valueKnob;
	float vm = vcoControlLevelVolumeView.valueMax;
	synthesizerOutput.vcoControlLevel = v / vm;
}

-(void)vcfCutOffFrequencyChanged:(id)sender {
	float v = vcfCutOffFrequencyVolumeView.valueKnob;
	float vm = vcfCutOffFrequencyVolumeView.valueMax;
	synthesizerOutput.vcfCutOffFrequency = 10000.0f * v / vm;
}
-(void)vcfResonanceChanged:(id)sender {
	float v = vcfResonanceVolumeView.valueKnob;
	float vm = vcfResonanceVolumeView.valueMax;
	synthesizerOutput.vcfResonance = 0.25f + 1.75f * v / vm;
}
-(void)vcfControlLevelChanged:(id)sender {
	float v = vcfControlLevelVolumeView.valueKnob;
	float vm = vcfControlLevelVolumeView.valueMax;
	synthesizerOutput.vcfControlLevel = v / vm;
}

-(void)vcaControlLevelChanged:(id)sender {
	float v = vcaControlLevelVolumeView.valueKnob;
	float vm = vcaControlLevelVolumeView.valueMax;
	synthesizerOutput.vcaControlLevel = v / vm;
}

-(void)egParamAttackChanged:(id)sender {
	float v = egParamAttackVolumeView.valueKnob;
	float vm = egParamAttackVolumeView.valueMax;
	synthesizerOutput.egParamAttack = 0.5f * v / vm;
}
-(void)egParamDecayChanged:(id)sender {
	float v = egParamDecayVolumeView.valueKnob;
	float vm = egParamDecayVolumeView.valueMax;
	synthesizerOutput.egParamDecay = v / vm;
}
-(void)egParamSustainChanged:(id)sender {
	float v = egParamSustainVolumeView.valueKnob;
	float vm = egParamSustainVolumeView.valueMax;
	synthesizerOutput.egParamSustain = v / vm;
}
-(void)egParamReleaseChanged:(id)sender {
	float v = egParamReleaseVolumeView.valueKnob;
	float vm = egParamReleaseVolumeView.valueMax;
	synthesizerOutput.egParamRelease = v / vm;
}

-(void)lfoWaveTypeChanged:(id)sender {
	synthesizerOutput.lfoWaveType = lfoWaveTypeVolumeView.valueKnob;
}
-(void)lfoPulseWidthChanged:(id)sender {
	float v = lfoPulseWidthVolumeView.valueKnob;
	float vm = lfoPulseWidthVolumeView.valueMax;
	synthesizerOutput.lfoPulseWidth = v / vm;
}
-(void)lfoFrequencyChanged:(id)sender {
	float v = lfoFrequencyVolumeView.valueKnob;
	float vm = lfoFrequencyVolumeView.valueMax;
	synthesizerOutput.lfoFrequency = 20.0f * v / vm;
}

-(void)echoDelayTimeChanged:(id)sender
{
	float v = echoDelayTimeVolumeView.valueKnob;
	float vm = echoDelayTimeVolumeView.valueMax;
	synthesizerOutput.echoDelayTime = v / vm;
}
-(void)echoDelayLevelChanged:(id)sender
{
	float v = echoDelayLevelVolumeView.valueKnob;
	float vm = echoDelayLevelVolumeView.valueMax;
	synthesizerOutput.echoDelayLevel = v / vm;
}

@end
