//
//  SynthVCF.m
//
//  Created by 成田 拓郎 on 10/05/01.
//  Copyright 2010 lovelyshark. All rights reserved.
//

#import "SynthVCF.h"

#define VCF_LFO_MAX     0.2f
#define VCF_M			2
#define VCF_N			2

@implementation SynthVCF


@synthesize cutOffFrequency;
@synthesize resonance;

//ローパスフィルタ
- (void)IIR_LPF:(float)fc Q:(float)Q a:(float*)a b:(float*)b
{
	// 係数計算
	float k = 4.0f  * M_PI * M_PI * fc * fc;
	float dinominator = 1.0f + (2.0f * M_PI * fc) / Q + k;
	a[1] = (2.0f * k - 2.0f) / dinominator;
	a[2] = (1.0f - (2.0f * M_PI * fc) / Q + k) / dinominator;
	k = k / dinominator;
	b[0] = k;
	b[1] = 2.0f * k;
	b[2] = k;
}

//このオブジェクトの演算関数
- (void) synthFunc:(int)length
{
	float *waveout_lfo  = (inputControlObject) ? inputControlObject.outputBuffer : [SynthObject dummyBuffer0];

	float *waveout_src  = (inputWaveObject) ? inputWaveObject.outputBuffer : [SynthObject dummyBuffer0];
	float *waveout_dest = outputBuffer;
	float *waveout_end  = waveout_dest + length;

	float outputSampleRate = [SynthObject outputSampleRate:-1.0f];

	//フィルタリング
	float freq, fc, level, a[VCF_N + 1], b[VCF_M + 1];
	float controlLevel_MAX = controlLevel * VCF_LFO_MAX;
	float M_PI_outputSampleRate = M_PI / outputSampleRate;
	int lpf_count = 0;
	while (waveout_dest != waveout_end) {
		//LFPの係数は250サンプルに1回計算する
		if (lpf_count == 0)
		{
			lpf_count = 249;

			//LFOによって変調されたカットオフ周波数
			freq = cutOffFrequency * (1.0f + controlLevel_MAX * (*waveout_lfo));
			if (freq < 100.0f)
				freq = 100.0f;
			// デジタルフィルタの周波数をアナログフィルタの周波数に変換
			fc = tanf(M_PI_outputSampleRate * freq) / (2.0f * M_PI);
			
			//ローパスフィルタの設計(10サンプルに１回計算し直す)
			[self IIR_LPF:fc Q:resonance a:a b:b];
		}
		else
			--lpf_count;
		++waveout_lfo;

		//フィルタリング
/*
		level = 0.0f;
		for (int m=0; m<=VCF_M; ++m) {
			level += b[m] * (*(waveout_src - m));
		}
		for (int n=1; n<=VCF_N; ++n) {
			level += - a[n] * (*(waveout_dest - n));
		}
*/
		level  =  b[0] * (*(waveout_src))
				+ b[1] * (*(waveout_src - 1))
				+ b[2] * (*(waveout_src - 2))
				- a[1] * (*(waveout_dest - 1))
				- a[2] * (*(waveout_dest - 2));

		//出力
		*waveout_dest = level;
		
		++waveout_src;
		++waveout_dest;		
	}
}

- (id) init
{
	if (self = [super init])
	{
		self.cutOffFrequency = 440.0f * 2.0f;
		self.resonance    = 1.0f;
	}
	
	return self;
}

- (void) dealloc
{
	
	[super dealloc];
}


@end
